import processing.core.PApplet;
import processing.core.PVector;

public class RadialAlphaSlider {

  public static final float TWO_PI = PApplet.TWO_PI;

  private PApplet p;

  float centerX = 300;
  float centerY = 300;
  float radius = 200;

  // Edge 1
  float angle1 = 0.1f;
  boolean highlight1 = false;

  // Edge 1
  float angle2 = 0.3f;
  boolean highlight2 = false;

  boolean highlightArc = false;

  float angleDistToEdges = 0.01f;

  public RadialAlphaSlider(PApplet p, float centerX, float centerY, float radius, float angle1,
  float angle2) {
    super();
    this.p = p;
    this.centerX = centerX;
    this.centerY = centerY;
    this.radius = radius;
    this.angle1 = angle1;
    this.angle2 = angle2;
  }

  public void draw() {


    p.noStroke();
    p.fill(200,200,200);
    p.ellipse(centerX, centerY, radius * 2, radius * 2);

    p.pushMatrix();
    p.translate(centerX, centerY);

    p.pushMatrix();
    for (int i = 0; i < 24; i ++) {
      p.stroke(0);
      p.strokeWeight(2);
      p.rotate(TWO_PI/24);
      p.line(radius + 4, 0, radius + 9, 0);
    }
    p.popMatrix();

    // Angle1 switch (see ArcTest2App)
    float angleToUse1 = angle1;
    if (angleToUse1 > angle2) {
      angleToUse1 = -(TWO_PI - angleToUse1);
    }

    // Arc
    if (highlightArc) {
      p.fill(50, 50, 50);
    } 
    else {
      p.fill(130,130,130);
    }
    p.noStroke();
    p.arc(0, 0, radius * 2, radius * 2, angleToUse1, angle2);

    p.fill(255);

    // Edge 1
    p.pushMatrix();
    p.rotate(angleToUse1);
    if (highlight1) {
      float x = radius * 0.7f;
      //      p.stroke(255, 0, 0, 150);
      p.triangle(x, 5, x + 5, 13, x + 10, 5);
      p.triangle(x, -5, x + 5, -13, x + 10, -5);
      //  p.stroke(255, 0, 0);
    } 
    else {
      //  p.stroke(0);
    }
    p.line(0, 0, radius, 0);
    p.popMatrix();

    // Edge 2
    p.pushMatrix();
    p.rotate(angle2);
    if (highlight2) {
      float x = radius * 0.7f;
      // p.stroke(255, 0, 0, 150);
      p.triangle(x, 5, x + 5, 13, x + 10, 5);
      p.triangle(x, -5, x + 5, -13, x + 10, -5);
      // p.stroke(255, 0, 0);
    } 
    else {
      // p.stroke(0);
    }
    p.line(0, 0, radius, 0);
    p.popMatrix();

    p.popMatrix();
  }

  public void onMoved(float x, float y) {
    // All interactions only if inside circle
    boolean insideCircle = PApplet.dist(centerX, centerY, x, y) < radius;

    // Angle between given pos and center
    PVector center = new PVector(centerX, centerY);
    PVector check = new PVector(x, y);
    float checkAngle = getAngleBetween(center, check);

    // Highlight if pos is close to edge 1
    if (Math.abs(angle1 - checkAngle) < angleDistToEdges && insideCircle) {
      highlight1 = false;
    } 
    else {
      highlight1 = false;
    }

    // Highlight if pos is close to edge 2
    if (Math.abs(angle2 - checkAngle) < angleDistToEdges && insideCircle) {
      highlight2 = false;
    } 
    else {
      highlight2 = false;
    }

    // If between both edges
    boolean inAngle1 = checkAngle > angle1 + angleDistToEdges;
    boolean inAngle2 = checkAngle < angle2 - angleDistToEdges;
    boolean inBetween = inAngle1 && inAngle2 && insideCircle;
    if (angle1 > angle2) {
      // Update condition if angle1 switch
      float angleToUse1 = -(TWO_PI - angle1);
      float mouseAngleToUse = -(TWO_PI - checkAngle);
      inAngle1 = Math.abs(mouseAngleToUse) < Math.abs(angleToUse1) - angleDistToEdges;
      inBetween = (inAngle1 || inAngle2) && insideCircle;
    }
    if (inBetween) {
      highlightArc = true;
    } 
    else {
      highlightArc = false;
    }
  }

  public void onDragged(float x, float y, float oldX, float oldY) {
    PVector center = new PVector(centerX, centerY);
    PVector check = new PVector(x, y);
    float checkAngle = getAngleBetween(center, check);

    if (highlight1) {
      angle1 = checkAngle;
    }
    if (highlight2) {
      angle2 = checkAngle;
    }

    if (highlightArc) {
      // Update both angles to move whole arc

        PVector oldCheck = new PVector(oldX, oldY);
      float oldAngle = getAngleBetween(center, oldCheck);
      float diffAngle = oldAngle - checkAngle;
      diffAngle = getRangedAngle(diffAngle);
      angle1 -= diffAngle;
      angle2 -= diffAngle;

      // NB: checkAngle only can be between 0 and TWO_PI, so it's fine,
      // but by subtracting a diffAngle the angles can be moved out of range
      angle1 = getRangedAngle(angle1);
      angle2 = getRangedAngle(angle2);
    }
  }

  protected float getRangedAngle(float angle) {
    float rangedAngle = angle;
    if (rangedAngle > TWO_PI) {
      rangedAngle = TWO_PI - rangedAngle;
    } 
    else {
      if (rangedAngle < 0) {
        rangedAngle = TWO_PI + rangedAngle;
      }
    }
    return rangedAngle;
  }

  protected static float getAngleBetween(PVector p1, PVector p2) {
    // Note: Raw values between point 1 and point 2 not valid, as they are are origin-based.
    PVector sub = PVector.sub(p2, p1);
    PVector xaxis = new PVector(1, 0);
    float angle = PVector.angleBetween(xaxis, sub);

    if (p2.y < p1.y) {
      angle = PApplet.TWO_PI - angle;
    }
    return angle;
  }
}

